/*
 * Copyright (C) 2008 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.qihoo.appstore.updatelib;

import android.app.Service;
import android.content.Intent;
import android.os.IBinder;
import android.util.Log;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * @author zhangyong6
 */
abstract class MultiIntentService extends Service {
    private boolean mRedelivery;

    private ExecutorService mExecutor;

    /**
     * Creates an IntentService. Invoked by your subclass's constructor.
     * 
     * @param name Used to name the worker thread, important only for debugging.
     */
    public MultiIntentService(String name) {
        super();
    }

    /**
     * Sets intent redelivery preferences. Usually called from the constructor
     * with your preferred semantics.
     * <p>
     * If enabled is true, {@link #onStartCommand(Intent, int, int)} will return
     * {@link Service#START_REDELIVER_INTENT}, so if this process dies before
     * {@link #onHandleIntent(Intent)} returns, the process will be restarted
     * and the intent redelivered. If multiple Intents have been sent, only the
     * most recent one is guaranteed to be redelivered.
     * <p>
     * If enabled is false (the default),
     * {@link #onStartCommand(Intent, int, int)} will return
     * {@link Service#START_NOT_STICKY}, and if the process dies, the Intent
     * dies along with it.
     */
    public void setIntentRedelivery(boolean enabled) {
        mRedelivery = enabled;
    }

    @Override
    public void onCreate() {
        super.onCreate();
        mExecutor = Executors.newCachedThreadPool();
    }

    @Override
    public void onStart(final Intent intent, final int startId) {
        mExecutor.submit(new Runnable() {
            @Override
            public void run() {
                try {
                    onHandleIntent(intent);
                } catch (Throwable e) {
                    Log.e("MultiIntentService", "onHandleIntent", e);
                } finally {
                    stopSelf(startId);
                }
            }
        });
    }

    /**
     * You should not override this method for your IntentService. Instead,
     * override {@link #onHandleIntent}, which the system calls when the
     * IntentService receives a start request.
     * 
     * @see android.app.Service#onStartCommand
     */
    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        onStart(intent, startId);
        return mRedelivery ? START_REDELIVER_INTENT : START_NOT_STICKY;
    }

    @Override
    public void onDestroy() {
        mExecutor.shutdown();
    }

    /**
     * Unless you provide binding for your service, you don't need to implement
     * this method, because the default implementation returns null.
     * 
     * @see android.app.Service#onBind
     */
    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }

    /**
     * This method is invoked on the worker thread with a request to process.
     * Only one Intent is processed at a time, but the processing happens on a
     * worker thread that runs independently from other application logic. So,
     * if this code takes a long time, it will hold up other requests to the
     * same IntentService, but it will not hold up anything else. When all
     * requests have been handled, the IntentService stops itself, so you should
     * not call {@link #stopSelf}.
     * 
     * @param intent The value passed to
     *            {@link android.content.Context#startService(Intent)}.
     */
    protected abstract void onHandleIntent(Intent intent);
}
